/*++

Copyright (c) 2010  Microsoft Corporation

Module Name:

    fbwfapi.h

Abstract:

    This is the exported header file that defines the prototype of the FBWF APIs
    and the structures used in these APIs

Environment:

    User mode

Revision History:

--*/

#ifndef __FBWF_API_H__
#define __FBWF_API_H__

#include <pshpack8.h>

#ifdef __cplusplus
extern "C" {
#endif

//
// 32-bit platforms
//
#define FBWF_DEFAULT_CACHE_THRESHOLD_32    128          /*  128 MB */
#define FBWF_MIN_CACHE_THRESHOLD_32        16           /*   16 MB */
#define FBWF_MAX_CACHE_THRESHOLD_32        1024         /* 1024 MB */

//
// 64-bit platforms
//
#define FBWF_DEFAULT_CACHE_THRESHOLD_64    256          /* 256 MB */
#define FBWF_MIN_CACHE_THRESHOLD_64        16           /*  16 MB */
#define FBWF_MAX_CACHE_THRESHOLD_64        131072       /* 128 GB */ 

#if defined(_WIN64)
#define FBWF_DEFAULT_CACHE_THRESHOLD    FBWF_DEFAULT_CACHE_THRESHOLD_64
#define FBWF_MIN_CACHE_THRESHOLD        FBWF_MIN_CACHE_THRESHOLD_64
#define FBWF_MAX_CACHE_THRESHOLD        FBWF_MAX_CACHE_THRESHOLD_64 
#else
#define FBWF_DEFAULT_CACHE_THRESHOLD    FBWF_DEFAULT_CACHE_THRESHOLD_32
#define FBWF_MIN_CACHE_THRESHOLD        FBWF_MIN_CACHE_THRESHOLD_32
#define FBWF_MAX_CACHE_THRESHOLD        FBWF_MAX_CACHE_THRESHOLD_32
#endif

typedef struct _FbwfCacheDetail {
    SIZE_T  cacheSize;               // size of cache used by the file
    ULONG   openHandleCount;         // number of currently opened handles
    ULONG   fileNameLength;          // file name length in bytes
    WCHAR   fileName[1];             // file name (may not be null-terminated)
} FbwfCacheDetail, *PFbwfCacheDetail;

typedef struct _FbwfMemoryUsage {
    SIZE_T  currCacheThreshold;     // current session cache threshold
    SIZE_T  nextCacheThreshold;     // next session cache threshold
    SIZE_T  dirStructure;           // memory used to store directory structure
    SIZE_T  fileData;               // memory used to cache file data
} FbwfMemoryUsage, *PFbwfMemoryUsage;

typedef struct _FbwfCacheDetailEx {
    ULONG size;              // must be set as sizeof(FbwfCacheDetailEx)
    LARGE_INTEGER cacheSize; // size of cache used by the file
    ULONG openHandleCount;   // number of currently opened handles
    ULONG fileNameLength;    // file name length in bytes
    WCHAR fileName[1];       // file name (null-terminated string)
} FbwfCacheDetailEx, *PFbwfCacheDetailEx;

typedef struct _FbwfMemoryUsageEx {
    ULONG size;                          // must be set as sizeof(FbwfMemoryUsageEx)
    ULONG reserved;                      // reserved
    LARGE_INTEGER currCacheThreshold;    // Cache Threshold in Bytes for the current session
    LARGE_INTEGER nextCacheThreshold;    // Cache Threshold in Bytes for the next session
    LARGE_INTEGER dirStructure;	         // Number of bytes used in cache for directories
    LARGE_INTEGER fileData;              // Number of bytes used in cache for file data
} FbwfMemoryUsageEx, * PFbwfMemoryUsageEx;

// Structure is identical to FILE_FS_SIZE_INFORMATION
// but we define here so user does not need to include header file.

typedef struct _FbwfFSInfo {
    LARGE_INTEGER totalAllocationUnits;
    LARGE_INTEGER availableAllocationUnits;
    ULONG sectorsPerAllocationUnit;
    ULONG bytesPerSector;
} FbwfFSInfo, *PFbwfFSInfo;

typedef struct _FbwfVolumeListEntry {
    DWORD   size;           // must be set as sizeof(FbwfVolumeListEntry)
    LPCWSTR volumeGuidName;
    LPCWSTR volumeName;
} FbwfVolumeListEntry, *PFbwfVolumeListEntry;

DECLARE_HANDLE(FBWF_FIND_VOLUME_HANDLE);
DECLARE_HANDLE(FBWF_FIND_FILE_HANDLE);

#if !defined(FBWFAPI_STRUCTURES_ONLY)

ULONG 
WINAPI 
FbwfEnableFilter(
    VOID
    );

ULONG 
WINAPI 
FbwfDisableFilter(
    VOID
    );

ULONG 
WINAPI 
FbwfProtectVolume(
    __in LPCWSTR volume
    );

ULONG 
WINAPI 
FbwfUnprotectVolume(
    __in LPCWSTR volume,
    __in ULONG removeExclusionList
    );

ULONG 
WINAPI 
FbwfSetCacheThreshold(
    __in ULONG thresholdInMB
    );

ULONG 
WINAPI 
FbwfSetCacheThresholdEx(
    __in LARGE_INTEGER thresholdInBytes
    );

ULONG 
WINAPI 
FbwfGetVolumeList(
    __in     ULONG currSession, 
    __in_opt PVOID volumeList, 
    __inout  PULONG size
    );

ULONG 
WINAPI 
FbwfFindFirstVolume(
    __in ULONG currSession,
    __out PFbwfVolumeListEntry volumeListEntry,
    __out FBWF_FIND_VOLUME_HANDLE * Handle
    );

ULONG
WINAPI
FbwfFindNextVolume(
    __in FBWF_FIND_VOLUME_HANDLE hFindVolume,
    __out PFbwfVolumeListEntry volumeListEntry
    );

ULONG
WINAPI
FbwfFindVolumeClose(
    __in FBWF_FIND_VOLUME_HANDLE hFindVolume
    );

ULONG 
WINAPI 
FbwfIsFilterEnabled(
    __out PULONG currentSession, 
    __out PULONG nextSession
    );

ULONG 
WINAPI 
FbwfIsVolumeProtected(
    __in LPCWSTR volume, 
    __out PULONG currentSession, 
    __out PULONG nextSession
    );

ULONG 
WINAPI 
FbwfGetMemoryUsage(
    __out PFbwfMemoryUsage usage
    );

ULONG 
WINAPI 
FbwfGetMemoryUsageEx(
    __out PFbwfMemoryUsageEx MemoryUsage
    );

ULONG 
WINAPI 
FbwfFindFirst(
    __in LPCWSTR volume, 
    __out PFbwfCacheDetail cacheDetail, 
    __inout PULONG size
    );

ULONG 
WINAPI 
FbwfFindNext(
    __out PFbwfCacheDetail cacheDetail, 
    __inout PULONG size
    );

ULONG 
WINAPI 
FbwfFindClose(
    VOID
    );

ULONG
WINAPI
FbwfFindFirstEx(
    __in LPCWSTR VolumeName,
    __out_bcount(*CacheDetailSize) PFbwfCacheDetailEx CacheDetail,
    __inout PULONG CacheDetailSize,
    __out FBWF_FIND_FILE_HANDLE * FindHandle
    );

ULONG
WINAPI
FbwfFindNextEx(
    __in FBWF_FIND_FILE_HANDLE FindHandle,
    __out_bcount(*CacheDetailSize) PFbwfCacheDetailEx CacheDetail,
    __inout PULONG CacheDetailSize
    );

ULONG
WINAPI
FbwfFindCloseEx(
    __in FBWF_FIND_FILE_HANDLE FindHandle
    );

ULONG 
WINAPI 
FbwfEnableCompression(
    VOID
    );

ULONG 
WINAPI 
FbwfDisableCompression(
    VOID
    );

ULONG 
WINAPI 
FbwfIsCompressionEnabled(
    __out PULONG currentSession, 
    __out PULONG nextSession
    );

ULONG 
WINAPI 
FbwfEnableCachePreAllocation(
    VOID
    );

ULONG 
WINAPI 
FbwfDisableCachePreAllocation(
    VOID
    );

ULONG 
WINAPI 
FbwfIsCachePreAllocationEnabled(
    __out PULONG currentSession, 
    __out PULONG nextSession
    );

ULONG 
WINAPI 
FbwfAddExclusion(
    __in LPCWSTR volume, 
    __in LPCWSTR path
    );

ULONG 
WINAPI 
FbwfRemoveExclusion(
    __in LPCWSTR volume, 
    __in LPCWSTR path
    );

ULONG 
WINAPI 
FbwfGetExclusionList(
    __in     LPCWSTR volume, 
    __in     ULONG currSession, 
    __in_opt PVOID exclusionList, 
    __inout  PULONG size
    );

ULONG 
WINAPI 
FbwfCommitFile(
    __in LPCWSTR volume, 
    __in LPCWSTR path
    );

ULONG 
WINAPI 
FbwfRestoreFile(
    __in LPCWSTR volume, 
    __in LPCWSTR path
    );

ULONG 
WINAPI 
FbwfCacheThresholdNotification(
    __in ULONG threshold, 
    __in HANDLE event
    );

ULONG 
WINAPI 
FbwfGetSizeDisplay(
    __out PULONG currentSession, 
    __out PULONG nextSession
    );

ULONG 
WINAPI 
FbwfSetSizeDisplay( 
    __in ULONG displayMode
    );

ULONG 
WINAPI 
FbwfGetActualSize( 
    __in LPCWSTR volume, 
    __out PFbwfFSInfo fsInfo
    );

ULONG 
WINAPI 
FbwfGetVirtualSize(
    __in LPCWSTR volume, 
    __out PFbwfFSInfo fsInfo
    );

HRESULT
WINAPI
FbwfGetDosName(
    __in LPCWSTR lpVolumeName,
    __inout LPWSTR lpDosName,
    __in DWORD dwDosNameBufferSize
    );

ULONG
WINAPI
FbwfInitializeNotifications(
    void
    );

ULONG
WINAPI
FbwfUninitializeNotifications(
    void
    );

#ifndef FBWFAPI_NO_DEPRECATE
// Deprecate functions to generate compile time errors. If you do not want
// this then you can #define FBWFAPI_NO_DEPRECATE before including this file
#ifdef DEPRECATE_SUPPORTED

#pragma deprecated(FbwfFindFirst)
#pragma deprecated(FbwfFindNext)
#pragma deprecated(FbwfFindClose)
#pragma deprecated(FbwfGetMemoryUsage)
#pragma deprecated(FbwfSetCacheThreshold)

#else // DEPRECATE_SUPPORTED

#undef FbwfFindFirst
#define FbwfFindFirst           FbwfFindFirst_instead_use_FbwfFindFirstEx;

#undef FbwfFindNext
#define FbwfFindNext            FbwfFindNext_instead_use_FbwfFindNextEx;

#undef FbwfFindClose
#define FbwfFindClose           FbwfFindClose_instead_use_FbwfFindCloseEx;

#undef FbwfGetMemoryUsage
#define FbwfGetMemoryUsage      FbwfGetMemoryUsage_instead_use_FbwfGetMemoryUsageEx;

#undef FbwfSetCacheThreshold
#define FbwfSetCacheThreshold   FbwfSetCacheThreshold_instead_use_FbwfSetCacheThresholdEx;

#endif  // DEPRECATE_SUPPORTED
#endif  // !FBWFAPI_NO_DEPRECATE

#endif /* FBWFAPI_STRUCTURES_ONLY */

#ifdef __cplusplus
}
#endif

#include <poppack.h>

#endif
